<?php

namespace Drupal\searchunifyd10\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\searchunifyd10\Services\CommonCalls;
use Drupal\searchunifyd10\Services\RestCalls;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class SuHelpController.Defines controller for Searchunifyd10 search results.
 *
 * @category Drupal
 * @package Drupal\searchunifyd10\Controller
 * @author searchunifyd10 <your.email@example.com>
 * @license GPL-2.0-or-later <https://www.gnu.org/licenses/gpl-2.0.html>
 * @link http://graztti.com
 * @php_version 7.2
 */
class SuResultController extends ControllerBase
{

  /**
   * A variable for service.
   *
   * @var \Drupal\searchunifyd10\CommonCalls
   */
  protected $ccall;

  /**
   * A variable for service.
   *
   * @var \Drupal\searchunifyd10\RestCalls
   */
  protected $rcall;

  /**
   * Defining cfactory variable for config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $cfactory;

  /**
   * Class constructor.
   *
   * @param \Drupal\searchunifyd10\CommonCalls $ccall
   *   The CommonCalls service.
   * @param \Drupal\searchunifyd10\RestCalls $rcall
   *   The RestCalls service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $cfactory
   *   The ConfigFactory service.
   */
  public function __construct(CommonCalls $ccall, RestCalls $rcall, ConfigFactoryInterface $cfactory)
  {
    $this->ccall    = $ccall;
    $this->rcall    = $rcall;
    $this->cfactory = $cfactory->get('sud10.configs');
  }

  /**
   * {@inheritdoc}
   *
   * A new instance of the class.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   A new instance of the class.
   */
  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('searchunifyd10.commonCalls'),
      $container->get('searchunifyd10.restCalls'),
      $container->get('config.factory')
    );
  }

  /**
   * Method to get result on frontend.
   *
   * @param string $dynamic_path
   *   The dynamic path.
   *
   * @return array
   *   An array containing theme information and module path.
   */
  public function content($dynamic_path)
  {
    $path = $dynamic_path;
    $repeater = $this->cfactory->get('repeater_data');
    $num_fields = $this->cfactory->get('num_fields');
    $urls = [];
    // print_r($repeater);
    for ($i = 0; $i < $num_fields; $i++) {
      $urls[$repeater[$i]['uid']] = $repeater[$i]['search_url'];
    }
    // print_r($urls);die();
    $matchedUid = NULL;
    foreach ($urls as $uid => $url) {
      if ($path == $url) {
        $matchedUid = $uid;
      }
    }

    $suConfigs           = [];
    $suConfigs['uid']    = $matchedUid;
    $suConfigs['token']  = $this->cfactory->get('access_token');
    $suConfigs['epoint'] = $this->cfactory->get('epoint');
    $suConfigs['cdn']    = $this->cfactory->get('cdn');

    return [
      '#theme' => 'su_results',
      '#data' => $suConfigs,
    ];
  }

  /**
   * Method to get result in API.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function resultsByPost()
  {

    $requestVals = \Drupal::request();
    $reqData     = $requestVals->getContent();
    $reqDataJson = json_decode($reqData);
    $postData    = (array) $reqDataJson;

    $configVals = $this->rcall->getConfigVals();

    if (($configVals['status'] == TRUE) && (isset($configVals['data'])) && (!empty($configVals['data']))) {
      $uid             = $configVals['data']['uid'];
      $token           = $configVals['data']['token'];
      $epoint          = $configVals['data']['epoint'];
      $reqUrl          = $epoint . 'api/v2_search/searchResults';
      $postData['uid'] = $uid;
      $resAry          = $this->rcall->exeHttpClient('POST', $reqUrl, $token, $postData);
      // print_r($reqUrl);die();
      if ($resAry['status'] == 200) {
        $reqResponse = json_decode($resAry['body']);
        $reqResponse = (array) $reqResponse;
        return new JsonResponse($reqResponse);
      } else {
        return new JsonResponse(['status' => 400, 'message' => 'failed']);
      }
    } else {
      return new JsonResponse(['status' => 400, 'message' => 'failed']);
    }
  }
  /**
   * Method to get params in API.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function getAPIParams()
  {
    $request = \Drupal::request();
    $requestData = $request->getContent();
    $header = getallheaders();
    // $reqDataJson = json_decode($requestData);
    // $postData    = (array) $reqDataJson;
    $configVals = $this->rcall->getConfigVals();

    if (($configVals['status'] == TRUE) && (isset($configVals['data'])) && (!empty($configVals['data']))) {
      $uid             = $configVals['data']['uid'];
      $token           = $configVals['data']['token'];
      $epoint          = $configVals['data']['epoint'];
      $reqUrl          = $epoint . 'mlService/su-gpt';
      $resAry          = $this->rcall->exeHttpClientSuGpt('POST', $reqUrl,$header, $token, $requestData);
      // print_r($reqUrl);die();
      if ($resAry['status'] == 200) {
        $reqResponse = json_decode($resAry['body']);
        $reqResponse = (array) $reqResponse;
        return new JsonResponse($reqResponse);
      } else {
        return new JsonResponse(['status' => 400, 'message' => 'failed']);
      }
    } else {
      return new JsonResponse(['status' => 400, 'message' => 'failed']);
    }
  }
}
