<?php

namespace Drupal\searchunifyd10\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\searchunifyd10\Services\CommonCalls;
use Drupal\searchunifyd10\Services\RestCalls;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * @file
 * Contains \Drupal\searchunifyd10\Form\SuConfigFrom.
 *
 * @category Drupal
 * @package Drupal\searchunifyd10\Controller
 * @author searchunifyd10 <your.email@example.com>
 * @license GPL-2.0-or-later <https://www.gnu.org/licenses/gpl-2.0.html>
 * @link http://graztti.com
 * @php_version 7.2
 */
/**
 * Class SuConfigForm.Defines form for Searchunifyd10 search results.
 */
class SuConfigForm extends ConfigFormBase {

  /**
   * A variable for service.
   *
   * @var \Drupal\searchunifyd10\CommonCalls
   */
  protected $ccall;

  /**
   * A variable for service.
   *
   * @var \Drupal\searchunifyd10\RestCalls
   */
  protected $rcall;

  /**
   * A Request stack instance.
   *
   * @var Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * Constructs a suConfigForm object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   A Request stack variable.
   * @param \Drupal\searchunifyd10\Services\CommonCalls $ccall
   *   Common Call Service variable.
   * @param \Drupal\searchunifyd10\Services\RestCalls $rcall
   *   Rest Call Service variable.
   */
  public function __construct(RequestStack $request, CommonCalls $ccall, RestCalls $rcall) {
    $this->request = $request;
    $this->ccall   = $ccall;
    $this->rcall   = $rcall;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   A container interface object.
   *
   * @return static
   *   A new instance of the class.
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('request_stack'),
          $container->get('searchunifyd10.commonCalls'),
          $container->get('searchunifyd10.restCalls')
      );
  }

  /**
   * Set Configuration Id.
   */
  protected function getEditableConfigNames() {
    return [
      'sud10.configs',
    ];
  }

  /**
   * Set Form Id.
   */
  public function getFormId() {
    return 'searchunifyd10_configs';
  }

  /**
   * {@inheritdoc}
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return parentbuildForm
   *   A new instance of the class.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $suConfigs = $this->config('sud10.configs');
    $devFields = $this->request->getCurrentRequest()->get('df');

    $form['hidden'] = [
      '#type' => 'hidden',
      '#attributes' => ['autocomplete' => 'off'],
    ];

    $form['body'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Configure searchunify access credentials'),
    ];

    $form['body']['cid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Id'),
      '#description' => $this->t('Client Id from searchunify app'),
      '#size' => 64,
      '#default_value' => $suConfigs->get('cid') ?? '',
      '#maxlength' => 200,
      '#required' => TRUE,
    ];

    $form['body']['cdn'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CDN'),
      '#description' => $this->t('CDN from searchunify app'),
      '#size' => 64,
      '#default_value' => $suConfigs->get('cdn') ?? '',
      '#maxlength' => 200,
      '#required' => TRUE,
    ];

    $form['body']['csecret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#description' => $this->t('Client secret from searchunify app'),
      '#size' => 64,
      '#default_value' => $suConfigs->get('csecret') ?? '',
      '#maxlength' => 200,
      '#required' => TRUE,
    ];

    $form['body']['epoint'] = [
      '#type' => 'url',
      '#title' => $this->t('Endpoint'),
      '#description' => $this->t('Endpoint URL i.e. https://xxxxxxxx.searchunify.com'),
      '#size' => 64,
      '#default_value' => $suConfigs->get('epoint') ?? '',
      '#maxlength' => 200,
      '#required' => TRUE,
    ];

    $form['body']['unm'] = [
      '#type' => 'password',
      '#title' => $this->t('Username'),
      '#description' => $this->t('Searchunify Username'),
      '#size' => 64,
      '#required' => TRUE,
      '#maxlength' => 200,
      '#attributes' => [
        'autocomplete' => 'off',
        'value' => (!empty($suConfigs->get('unm'))) ? $suConfigs->get('unm') : '',
      ],
    ];

    $form['body']['psw'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#description' => $this->t('Searchunify Password'),
      '#size' => 64,
      '#maxlength' => 200,
      '#required' => TRUE,
      '#attributes' => [
        'autocomplete' => 'off',
        'value' => (!empty($suConfigs->get('psw'))) ? $suConfigs->get('psw') : '',
      ],
    ];

    $form['#tree'] = TRUE;

    $num_fields = $this->config('sud10.configs')->get('num_fields');
    if ($num_fields == '' || $num_fields == NULL) {
      $this->config('sud10.configs')->set('num_fields', 1)->save();
    }

    $form['body']['items_fieldset'] = [
      '#type' => 'fieldset',
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => ['repeater-fieldset-wrapper'],
      ],
    ];

    $form['body']['items_fieldset']['actions'] = [
      '#type' => 'actions',
      '#attributes' => [
        'class' => ['repeater-fieldset'],
      ],
    ];
    $form['body']['items_fieldset']['actions']['add_item'] = [
      '#type' => 'submit',
      '#value' => $this->t('<span>&#43;</span> Add'),
      '#submit' => ['::addMore'],
      '#ajax' => [
        'callback' => '::addItemCallback',
        'wrapper' => 'items-fieldset-wrapper',
      ],
      '#attributes' => [
        'class' => ['add-btn'],
      ],
      '#limit_validation_errors' => [],
    ];

    for ($i = 0; $i < $num_fields; $i++) {
      $form['body']['items_fieldset'][$i] = $this->getItemFields($i, $i == 0, $form_state);
    }

    if ($devFields == '1') {
      $form['body']['atkn'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Access_Token'),
        '#default_value' => $suConfigs->get('access_token') ?? '',
        '#attributes' => ['disabled' => 'disabled'],
      ];

      $form['body']['atkn_expry'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Access_Token_Expiry ( UTC )'),
        '#default_value' => $suConfigs->get('access_token_expiry') ?? '',
        '#attributes' => ['disabled' => 'disabled'],
      ];

      $form['body']['rtkn'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Refresh_Token'),
        '#default_value' => $suConfigs->get('refresh_token') ?? '',
        '#attributes' => ['disabled' => 'disabled'],
      ];

      $form['body']['rtkn_expry'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Refresh_Token_Expiry ( UTC )'),
        '#default_value' => $suConfigs->get('refresh_token_expiry') ?? '',
        '#attributes' => ['disabled' => 'disabled'],
      ];

      $form['body']['tkn'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Response Detail'),
        '#description' => $this->t('Detail JSON of generated token'),
        '#default_value' => $suConfigs->get('tkn') ?? '',
        '#attributes' => ['disabled' => 'disabled'],
      ];
    }

    $form['body']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Configuration'),
      '#prefix' => '<div class="form-action">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
    ];

    $form['footer'] = [
      '#type' => 'markup',
      '#markup' => $this->ccall->grazittiHeader(),
    ];

    $form['#attributes'] = ['autocomplete' => 'off'];
    $form['#attached']['library'][] = 'searchunifyd10/searchunifyd10';

    return $form;
  }

  /**
   * Get the form fields for an individual item in the repeater.
   *
   * @param int $i
   *   The index of the item.
   * @param bool $isFirstItem
   *   A boolean indicating whether this is the first item.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   An associative array containing the form fields for the item.
   */
  public function getItemFields($i, $isFirstItem, FormStateInterface &$form_state) {
    $repeater = $this->config('sud10.configs')->get('repeater_data');

    if (!$isFirstItem) {
      return [
        'wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['repeater-div'],
          ],
          'uid' => [
            '#type' => 'textfield',
            '#title' => $this->t('UID'),
            '#description' => $this->t('Unique identification key'),
            '#size' => 64,
            '#default_value' => $repeater[$i]['uid'] ?? '',
            '#maxlength' => 200,
            '#required' => TRUE,
            '#attributes' => [
              'autocomplete' => 'off',
              'data-id' => $i,
              'data-uid' => 'default-' . $repeater[$i]['uid'],
            ],
          ],
          'search_url' => [
            '#type' => 'textfield',
            '#title' => $this->t('Search URL'),
            '#description' => $this->t('Search URL (/searchunify/{searchurl})'),
            '#size' => 64,
            '#default_value' => $repeater[$i]['search_url'] ?? '',
            '#maxlength' => 200,
            '#required' => TRUE,
            '#attributes' => [
              'autocomplete' => 'off',
              'data-id' => $i,
            ],
          ],

          'remove_item' => [
            '#type' => 'submit',
            '#value' => $this->t('Remove'),
            '#name' => 'remove_item_' . $i,
            '#attributes' => [
              'class' => ['remove-item-btn'],
              'data-index' => $i,
            ],
            '#submit' => ['::removeOne'],
            '#limit_validation_errors' => [],
            '#ajax' => [
              'callback' => '::removeItemAjaxCallback',
              'wrapper' => 'items-fieldset-wrapper',
            ],
          ],
        ],
      ];
    }
    else {
      return [
        'wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['repeater-div'],
          ],
          'uid' => [
            '#type' => 'textfield',
            '#title' => $this->t('UID'),
            '#description' => $this->t('Unique identification key'),
            '#size' => 64,
            '#default_value' => $repeater[$i]['uid'] ?? '',
            '#maxlength' => 200,
            '#required' => TRUE,
            '#attributes' => ['autocomplete' => 'off'],
          ],
          'search_url' => [
            '#type' => 'textfield',
            '#title' => $this->t('Search URL'),
            '#description' => $this->t('Search URL (/searchunify/{searchurl})'),
            '#size' => 64,
            '#default_value' => $repeater[$i]['search_url'] ?? '',
            '#maxlength' => 200,
            '#required' => TRUE,
            '#attributes' => ['autocomplete' => 'off'],
          ],
        ],
      ];
    }
  }

  /**
   * Submit handler for adding more items to the repeater.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function addMore(array &$form, FormStateInterface $form_state) {
    $num_fields = \Drupal::config('sud10.configs')->get('num_fields');
    // print_r($num_fields);die();
    \Drupal::configFactory()->getEditable('sud10.configs')->set("num_fields", $num_fields + 1)->save();
    $form_state->setRebuild();
  }

  /**
   * Submit handler for remove one items to the repeater.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function removeOne(array &$form, FormStateInterface &$form_state) {
    $suConfigs = $this->config('sud10.configs');
    $triggering_element = $form_state->getTriggeringElement();
    $num_fields = $suConfigs->get('num_fields');
    $remove_index = str_replace('remove_item_', '', $triggering_element['#name']);
    $repeater_data = $suConfigs->get('repeater_data');

    // Check if the repeater_data for this index exists and remove it.
    if (isset($repeater_data[$remove_index])) {
      unset($repeater_data[$remove_index]);
      $suConfigs->set('repeater_data', array_values($repeater_data))->save();
    }

    // Decrement the num_fields value.
    if ($num_fields > 1) {
      $suConfigs->set("num_fields", $num_fields - 1)->save();
    }

    // Unset the values from form.
    $user_input = $form_state->getUserInput();
    if (isset($user_input['body']['items_fieldset'])) {
      // Remove the value at the specified index and re-index the array.
      array_splice($user_input['body']['items_fieldset'], $remove_index, 1);
      $user_input['body']['items_fieldset'] = array_values($user_input['body']['items_fieldset']);
      // Set the updated values back to the form state.
      $form_state->setUserInput($user_input);
    }

    // Rebuild the form.
    $form_state->setRebuild();
    return TRUE;
  }

  /**
   * Callback for adding more items to the repeater.
   *
   * @param array $form
   *   The form structure.
   *
   * @return array
   *   The updated form structure.
   */
  public function addItemCallback(array &$form) {
    return $form['body']['items_fieldset'];
  }

  /**
   * Callback for removing items from the repeater.
   *
   * @param array $form
   *   The form structure.
   *
   * @return array
   *   The updated form structure.
   */
  public function removeItemAjaxCallback(array &$form) {
    return $form['body']['items_fieldset'];
  }

  /**
   * {@inheritdoc}
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $configData = [];
    $repeater_data = [];

    $values = $form_state->cleanValues()->getValues();
    if (isset($values['body'])) {
      $configData['cid'] = $values['body']['cid'];
      $configData['cdn'] = $values['body']['cdn'];
      $configData['csecret'] = $values['body']['csecret'];
      $configData['epoint'] = $values['body']['epoint'];
      $configData['unm'] = $values['body']['unm'];
      $configData['psw'] = $values['body']['psw'];
      // Corrected key.
      $configData['access_token'] = $values['access_token'];
      // Corrected key.
      $configData['access_token_expiry'] = $values['access_token_expiry'];
      // Corrected key.
      $configData['refresh_token'] = $values['refresh_token'];
      // Corrected key.
      $configData['refresh_token_expiry'] = $values['refresh_token_expiry'];
      // Corrected key.
      $configData['tkn'] = $values['tkn'];

      foreach ($values['body']['items_fieldset'] as $key => $val) {
        if (isset($val['wrapper']['uid']) && isset($val['wrapper']['search_url'])) {
          $uid = $val['wrapper']['uid'];
          $search_url = $val['wrapper']['search_url'];
          $repeater_data[] = [
            'uid' => $uid,
            'search_url' => $search_url,
          ];
        }
      }

      try {
        foreach ($configData as $key => $val) {
          $this->config('sud10.configs')->set($key, $val)->save();
        }
        $this->config('sud10.configs')->set('repeater_data', $repeater_data)->save();

        $this->messenger()->addStatus($this->t('Configurations saved successfully'), 'success');
      }
      catch (Exception $e) {
        $this->messenger()->addError($this->t('An error occurred while saving the configurations.'), 'error');
      }
    }
  }

  /**
   * {@inheritdoc}
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $cid = $form_state->getValue(['body', 'cid']);
    $csecret = $form_state->getValue(['body', 'csecret']);
    $epoint = $form_state->getValue(['body', 'epoint']);
    $unm = $form_state->getValue(['body', 'unm']);
    $psw = $form_state->getValue(['body', 'psw']);

    if ((!empty($cid)) && (!empty($csecret)) && (!empty($unm)) && (!empty($psw)) && (!empty($epoint))) {
      $authUrl  = $epoint . '/oauth/token';
      $authCode = base64_encode($cid . ':' . $csecret);
      $authbody = [
        'grant_type'    => 'password',
        'username'      => $unm,
        'password'      => $psw,
      ];
      try {
        $tokenRes = $this->rcall->genOauthToken($authUrl, $authCode, $authbody);
        if (($tokenRes['status'] == '200') && (isset($tokenRes['body']))  &&  (!empty($tokenRes['body']))) {
          $resData = json_decode($tokenRes['body']);
          // Set - Access Token.
          $aToken = '';
          if ((isset($resData->accessToken)) && (!empty($resData->accessToken))) {
            $aToken = $resData->accessToken;
          }
          $form_state->setValue('access_token', $aToken);

          // Set - Access Token Expiry.
          $aTokenExpiresAt = '';
          if ((isset($resData->accessTokenExpiresAt)) && (!empty($resData->accessTokenExpiresAt))) {
            $aTokenExpiresAt = $resData->accessTokenExpiresAt;
          }
          $form_state->setValue('access_token_expiry', $aTokenExpiresAt);

          // Set - Refresh Token.
          $rToken = '';
          if ((isset($resData->refreshToken)) && (!empty($resData->refreshToken))) {
            $rToken = $resData->refreshToken;
          }
          $form_state->setValue('refresh_token', $rToken);

          // Set - Refresh Token Expiry.
          $rTokenExpiresAt = '';
          if ((isset($resData->refreshTokenExpiresAt)) && (!empty($resData->refreshTokenExpiresAt))) {
            $rTokenExpiresAt = $resData->refreshTokenExpiresAt;
          }
          $form_state->setValue('refresh_token_expiry', $rTokenExpiresAt);

          // Set - Full Token Response.
          $form_state->setValue('tkn', $tokenRes['body']);
        }
        else {
          $form_state->setErrorByName('message', $this->t('Invalid configurations to connect with searchunify.'));
          // $form_state->setRebuild();
        }
      }
      catch (Exception $e) {
        // Echo 'Caught exception: ',  $e->getMessage(), "\n";.
        $form_state->setErrorByName('message', $this->t('Invalid Configurations passed'));
      }
    }
    else {
      $form_state->setErrorByName('message', $this->t('All required fields are mandatory.'));
    }
  }

}
