<?php

namespace Drupal\searchunifyd10\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\RequestException;

/**
 * Restcalls Class Doc Comment.
 *
 * Endpoint Helper to retrieve application-wide
 * URLs based on the active web instance.
 *
 * @category Class
 * @package Restcalls
 * @author Your Name <your.email@example.com>
 * @license GNU General Public License version 2 or later; see LICENSE
 * @link http://graztti.com
 *
 * @since 1.0.1
 */
class RestCalls {

  /**
   * Defining Client id string variable.
   *
   * @var cid
   */
  protected $cid = NULL;

  /**
   * Defining Client Secret string variable.
   *
   * @var csecret
   */
  protected $csecret = NULL;

  /**
   * Defining UID string variable.
   *
   * @var uid
   */
  protected $uid = NULL;

  /**
   * Defining Endpoint string variable.
   *
   * @var epoint
   */
  protected $epoint = NULL;

  /**
   * Defining SU Username string variable.
   *
   * @var unm
   */
  protected $unm = NULL;

  /**
   * Defining SU Password string variable.
   *
   * @var psw
   */
  protected $psw = TRUE;

  /**
   * Defining Access Token string variable.
   *
   * @var accsestoken
   */
  protected $accsestoken = '';

  /**
   * Defining Access Token Expiry string variable.
   *
   * @var accsestokenExpiry
   */
  protected $accsestokenExpiry = '';

  /**
   * Defining Refresh Token string variable.
   *
   * @var refreshToken
   */
  protected $refreshToken = '';

  /**
   * Defining Refresh Token Expiry string variable.
   *
   * @var refreshTokenExpiry
   */
  protected $refreshTokenExpiry = '';

  /**
   * Defining sslVerify boolean variable for ssl issue in curl.
   *
   * @var sslVerify
   */
  protected $sslVerify = TRUE;

  /**
   * Defining httpClient variable for  curl.
   *
   * @var \Drupal\http_client_manager\HttpClientInterface
   */
  protected $httpClient;

  /**
   * Defining cfactory variable for config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $cfactory;

  /**
   * Defining suConfigs property for configuration values.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $suConfigs;

  /**
   * The constructor.
   *
   * @param \Drupal\http_client_manager\Client $httpClient
   *   HTTP Client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $cfactory
   *   The Config Factory.
   */
  public function __construct(Client $httpClient, ConfigFactoryInterface $cfactory) {
    $this->sslVerify  = TRUE;
    $this->httpClient = $httpClient;
    $this->cfactory   = $cfactory->get('sud10.configs');

    // $repeater = $this->cfactory->get('repeater_data');
    $request = \Drupal::request();
    $content = $request->getContent();
    $data = json_decode($content, TRUE);

    $uid = NULL;
    if (isset($data['uid'])) {
      $uid = $data['uid'];
    }

    $this->cid                = $this->cfactory->get('cid');
    $this->csecret            = $this->cfactory->get('csecret');
    $this->uid                = $uid;
    $this->epoint             = $this->cfactory->get('epoint');
    $this->unm                = $this->cfactory->get('unm');
    $this->psw                = $this->cfactory->get('psw');
    $this->accsestoken        = $this->cfactory->get('access_token');
    $this->accsestokenExpiry  = $this->cfactory->get('access_token_expiry');
    $this->refreshToken       = $this->cfactory->get('refresh_token');
    $this->refreshTokenExpiry = $this->cfactory->get('refresh_token_expiry');
    $this->suConfigs          = $cfactory->getEditable('sud10.configs');
  }

  /**
   * Method to generate Access Token from Searchunify Server.
   *
   * @param string $authUrl
   *   endpoint URL to execute.
   * @param string $authCode
   *   Authorization code.
   * @param string $authbody
   *   JSON string to pass as the body of the request.
   *
   * @return array
   *   An array containing the status and body.
   */
  public function genOauthToken($authUrl, $authCode, $authbody) {
    $reqParam = [];
    $reqParam['verify'] = $this->sslVerify;
    $reqParam['headers'] = [
      'Authorization' => 'Basic ' . $authCode,
      'Content-Type' => 'application/x-www-form-urlencoded',
      'cache-control' => 'no-cache',
    ];
    $reqParam['form_params'] = $authbody;
    $resAry = [];
    try {
      $client = new Client();
      $exeReq = $client->request('POST', $authUrl, $reqParam);
      $resAry['status'] = $exeReq->getStatusCode();
      $resAry['body'] = $exeReq->getBody()->getContents();
      return $resAry;
    }
    catch (RequestException $e) {
      $resAry['status'] = 400;
      $resAry['message'] = 'Invalid Request';
      return $resAry;
    }
  }

  /**
   * Method to generate Access Token by Refresh Token.
   *
   * @param string $authUrl
   *   endpoint URL to execute.
   * @param string $authCode
   *   Authorization code.
   * @param string $authbody
   *   JSON string to pass as body of request.
   *
   * @return array
   *   An array containing the status code and response body.
   */
  public function genOauthTokenByRefreshToken($authUrl, $authCode, $authbody) {
    $reqParam = [];
    $reqParam['verify'] = $this->sslVerify;
    $reqParam['headers'] = [
      'Authorization' => 'Basic ' . $authCode,
      'Content-Type' => 'application/x-www-form-urlencoded',
      'cache-control' => 'no-cache',
    ];
    $reqParam['form_params'] = $authbody;
    $resAry = [];
    try {
      $client = new Client();
      $exeReq = $client->request('POST', $authUrl, $reqParam);
      $resAry['status'] = $exeReq->getStatusCode();
      $resAry['body'] = $exeReq->getBody()->getContents();
    }
    catch (ConnectException $e) {
      $resAry['status'] = 400;
      $resAry['message'] = 'Invalid Request';
      return $resAry;
    }
    return $resAry;
  }

  /**
   * Method to get Configuration Values.
   *
   * @return array
   *   An array containing configuration values.
   *   - 'status': boolean, the status of the operation.
   *   - 'data': array, configuration data.
   */
  public function getConfigVals() {
    // Check : If "Access Token Expiry" is not empty.
    if (!empty($this->accsestokenExpiry)) {
      $accessTokenExpiryVal = strtotime($this->accsestokenExpiry);
      $currentTimestampVal  = time();
      // Check : If "Access_Token_Expiry" is greater than "Current_Timestamp".
      if ($accessTokenExpiryVal >= $currentTimestampVal) {
        return [
          'status'    => TRUE,
          'data'      => [
            'uid'       => $this->uid,
            'token'     => $this->accsestoken,
            'epoint'    => $this->epoint,
          ],
        ];
      }
      else {
        // Regenerate "Access Token" by using "Refresh token".
        $authUrl = $this->epoint . '/oauth/token';
        $authCode = base64_encode($this->cid . ':' . $this->csecret);
        $refreshTokenExpiryVal = strtotime($this->refreshTokenExpiry);
        if ($refreshTokenExpiryVal >= $currentTimestampVal) {
          $authbody = [
            'grant_type'    => 'refresh_token',
            'refresh_token' => $this->refreshToken,
          ];
          $tokenResult = $this->genOauthTokenByRefreshToken($authUrl, $authCode, $authbody);
          if (($tokenResult['status'] == '200') && (isset($tokenResult['body']))  &&  (!empty($tokenResult['body']))) {
            $tokenResData = json_decode($tokenResult['body']);
            return $this->updateConfigVals($tokenResData);
          }
          else {
            return ['status' => FALSE, 'data' => []];
          }
        }
        else {
          $authbody = [
            'grant_type'    => 'password',
            'username'      => $this->unm,
            'password'      => $this->psw,
          ];
          $tokenResult = $this->genOauthToken($authUrl, $authCode, $authbody);
          if (($tokenResult['status'] == '200') && (isset($tokenResult['body']))  &&  (!empty($tokenResult['body']))) {
            $tokenResData = json_decode($tokenResult['body']);
            return $this->updateConfigVals($tokenResData);
          }
          else {
            return ['status' => FALSE, 'data' => []];
          }
        }
      }
    }
    else {
      return ['status' => FALSE, 'data' => []];
    }
  }

  /**
   * Method to update tokens & expiry time.
   *
   * @param object $tokenResData
   *   Token response data generated by CURL request.
   *
   * @return array
   *   An array with status and data.
   */
  public function updateConfigVals($tokenResData) {
    // Set - Access Token.
    $aToken = '';
    if ((isset($tokenResData->accessToken)) && (!empty($tokenResData->accessToken))) {
      $aToken = $tokenResData->accessToken;
    }
    $this->suConfigs->set('access_token', $aToken)->save();

    // Set - Access Token Expiry.
    $aTokenExpiresAt = '';
    if ((isset($tokenResData->accessTokenExpiresAt)) && (!empty($tokenResData->accessTokenExpiresAt))) {
      $aTokenExpiresAt = $tokenResData->accessTokenExpiresAt;
    }
    $this->suConfigs->set('access_token_expiry', $aTokenExpiresAt)->save();

    // Set - Refresh Token.
    $rToken = '';
    if ((isset($tokenResData->refreshToken)) && (!empty($tokenResData->refreshToken))) {
      $rToken = $tokenResData->refreshToken;
    }
    $this->suConfigs->set('refresh_token', $rToken)->save();

    // Set - Refresh Token Expiry.
    $rTokenExpiresAt = '';
    if ((isset($tokenResData->refreshTokenExpiresAt)) && (!empty($tokenResData->refreshTokenExpiresAt))) {
      $rTokenExpiresAt = $tokenResData->refreshTokenExpiresAt;
    }
    $this->suConfigs->set('refresh_token_expiry', $rTokenExpiresAt)->save();

    // Set - Full Token Response.
    $this->suConfigs->set('tkn', $tokenResData);

    return [
      'status'    => TRUE,
      'data'      => [
        'uid'       => $this->uid,
        'token'     => $aToken,
        'epoint'    => $this->epoint,
      ],
    ];
  }

  /**
   * Method to execute cURL requests.
   *
   * @param string $reqType
   *   The HTTP request type (GET, POST, etc.).
   * @param string $reqUrl
   *   The URL to make the HTTP request to.
   * @param string $authToken
   *   The authorization token to include in the request header.
   * @param string $reqbody
   *   The body of the HTTP request.
   *
   * @return array
   *   An array containing the HTTP status code and the response body.
   */
  public function exeHttpClient($reqType, $reqUrl, $authToken, $reqbody) {
    $reqParam = [];
    $reqParam['verify'] = $this->sslVerify;
    $reqParam['headers'] = [
      'Authorization' => 'Bearer ' . $authToken,
      'Content-Type' => 'application/x-www-form-urlencoded',
      'cache-control' => 'no-cache',
    ];
    $reqParam['form_params'] = $reqbody;
    $resAry = [];
    try {
      $client = new Client();
      $exeReq = $client->request($reqType, $reqUrl, $reqParam);
      $resAry['status'] = $exeReq->getStatusCode();
      $resAry['body'] = $exeReq->getBody()->getContents();
      return $resAry;
    }
    catch (ConnectException $e) {
      $resAry['status'] = 400;
      $resAry['message'] = 'Invalid Request';
      return $resAry;
    }
  }

    /**
   * Method to execute cURL requests.
   *
   * @param string $reqType
   *   The HTTP request type (GET, POST, etc.).
   * @param string $reqUrl
   *   The URL to make the HTTP request to.
   * @param string $authToken
   *   The authorization token to include in the request header.
   * @param string $reqbody
   *   The body of the HTTP request.
   *
   * @return array
   *   An array containing the HTTP status code and the response body.
   */
  public function exeHttpClientSuGpt($reqType, $reqUrl, $headers, $authToken, $reqbody) {
    $reqParam = [];
    $reqParam['verify'] = $this->sslVerify;
    $reqParam['headers'] = [
      'token' => $authToken,
      'Content-Type' => 'application/json',
      'cache-control' => 'no-cache',
    ];
    $reqParam['headers'] = array_merge($reqParam['headers'], $headers);
    $reqParam['form_params'] = $reqbody;
    $resAry = [];
    try {
      $client = new Client();
      $exeReq = $client->request($reqType, $reqUrl, $reqParam);
      $resAry['status'] = $exeReq->getStatusCode();
      $resAry['body'] = $exeReq->getBody()->getContents();
      return $resAry;
    }
    catch (ConnectException $e) {
      $resAry['status'] = 400;
      $resAry['message'] = 'Invalid Request';
      return $resAry;
    }
  }

}
